@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2008 Frank Denis. All rights reserved.
+ * Copyright (c) 2004-2009 Frank Denis. All rights reserved.
  * 
  * This crucial part of UCARP is derived from the OpenBSD project.
  * Original copyright follows. 
@@ -242,13 +242,21 @@ static void carp_send_ad(struct carp_softc *sc)
     eh.ether_shost[4] = 0x00;
     eh.ether_shost[5] = vhid;
     
-    eh.ether_dhost[0] = 0x01;
-    eh.ether_dhost[1] = 0x00;
-    eh.ether_dhost[2] = 0x5e;
-    eh.ether_dhost[3] = 0x00;
-    eh.ether_dhost[4] = 0x00;
-    eh.ether_dhost[5] = 0x12;        
-    
+    if (no_mcast) {
+        eh.ether_dhost[0] = 0xff;
+        eh.ether_dhost[1] = 0xff;
+        eh.ether_dhost[2] = 0xff;
+        eh.ether_dhost[3] = 0xff;
+        eh.ether_dhost[4] = 0xff;
+        eh.ether_dhost[5] = 0xff;        
+    } else {
+        eh.ether_dhost[0] = 0x01;
+        eh.ether_dhost[1] = 0x00;
+        eh.ether_dhost[2] = 0x5e;
+        eh.ether_dhost[3] = 0x00;
+        eh.ether_dhost[4] = 0x00;
+        eh.ether_dhost[5] = 0x12;        
+    }    
     eh.ether_type = htons(ETHERTYPE_IP);    
     
     memcpy(pkt, &eh, sizeof eh);
@@ -569,10 +577,10 @@ static void packethandler(unsigned char *dummy,
             if (timercmp(&sc_tv, &ch_tv, >) ||
                 (timercmp(&sc_tv, &ch_tv, ==) &&
                  iphead.ip_src.s_addr < srcip.s_addr)) {
-		/* To be really sure the new master knows that is
-		 * has to reassert control of the VIP */
-		carp_send_ad(&sc);
-		
+        /* To be really sure the new master knows that is
+         * has to reassert control of the VIP */
+        carp_send_ad(&sc);
+        
                 carp_set_state(&sc, BACKUP);
                 carp_setrun(&sc, 0);
                 logfile(LOG_WARNING, _("Preferred master advertised: "
@@ -587,13 +595,13 @@ static void packethandler(unsigned char *dummy,
              */
             if (timercmp(&sc_tv, &ch_tv, <) ||
                 (timercmp(&sc_tv, &ch_tv, ==) &&
-		 iphead.ip_src.s_addr > srcip.s_addr)) {
-		gratuitous_arp(dev_desc_fd);
-		sc.sc_delayed_arp = 2; /* and yet another in 2 ticks */
-		logfile(LOG_WARNING, _("Non-preferred master advertising: "
-				       "reasserting control of VIP with another gratuitous arp"));
+         iphead.ip_src.s_addr > srcip.s_addr)) {
+        gratuitous_arp(dev_desc_fd);
+        sc.sc_delayed_arp = 2; /* and yet another in 2 ticks */
+        logfile(LOG_WARNING, _("Non-preferred master advertising: "
+                       "reasserting control of VIP with another gratuitous arp"));
             }
-	    break;
+        break;
         case BACKUP:
             /*
              * If we're pre-empting masters who advertise slower than us,
@@ -649,11 +657,11 @@ RETSIGTYPE sighandler_usr(const int sig)
 {
     switch (sig) {
     case SIGUSR1:
-	received_signal=1;
-	break;
+    received_signal=1;
+    break;
     case SIGUSR2:
-	received_signal=2;
-	break;
+    received_signal=2;
+    break;
     }
 }
 
@@ -757,15 +765,17 @@ int docarp(void)
                 interface == NULL ? "-" : interface, strerror(errno));
         return -1;
     }
-    memset(&req_add, 0, sizeof req_add);
-    req_add.imr_multiaddr.s_addr = inet_addr("224.0.0.18");
-    req_add.imr_interface.s_addr = srcip.s_addr;
-    if (setsockopt(fd, IPPROTO_IP, IP_ADD_MEMBERSHIP,
-		   &req_add, sizeof req_add) < 0) {
-	logfile(LOG_ERR, "Can't do IP_ADD_MEMBERSHIP errno=%s (%d)",
-		strerror(errno), errno);
-	close(fd);
-	return -1;
+    if (!no_mcast) {
+        memset(&req_add, 0, sizeof req_add);
+        req_add.imr_multiaddr.s_addr = inet_addr("224.0.0.18");
+        req_add.imr_interface.s_addr = srcip.s_addr;
+        if (setsockopt(fd, IPPROTO_IP, IP_ADD_MEMBERSHIP,
+                       &req_add, sizeof req_add) < 0) {
+            logfile(LOG_ERR, "Can't do IP_ADD_MEMBERSHIP errno=%s (%d)",
+                    strerror(errno), errno);
+            close(fd);
+            return -1;
+        }
     }
 #ifdef SIOCGIFFLAGS    
     if (strlen(interface) >= sizeof iface.ifr_name) {
@@ -780,18 +790,18 @@ int docarp(void)
             break;
         }
         if ((iface.ifr_flags & IFF_RUNNING) == 0) {
-	    if (ignoreifstate == 0) {
-		carp_set_state(&sc, BACKUP);
-		sc.sc_ad_tmo.tv_sec = 0;
-		sc.sc_ad_tmo.tv_usec = 0;
-		sc.sc_md_tmo.tv_sec = 0;
-		sc.sc_md6_tmo.tv_usec = 0;
-        	if (iface_running) {
-		    iface_running = 0;
-		}
-		sleep(SECONDS_TO_WAIT_AFTER_INTERFACE_IS_DOWN);
-		continue;
-	    }
+        if (ignoreifstate == 0) {
+        carp_set_state(&sc, BACKUP);
+        sc.sc_ad_tmo.tv_sec = 0;
+        sc.sc_ad_tmo.tv_usec = 0;
+        sc.sc_md_tmo.tv_sec = 0;
+        sc.sc_md6_tmo.tv_usec = 0;
+            if (iface_running) {
+            iface_running = 0;
+        }
+        sleep(SECONDS_TO_WAIT_AFTER_INTERFACE_IS_DOWN);
+        continue;
+        }
         } else {
             if (!iface_running) {
                 iface_running = 1;
@@ -801,28 +811,28 @@ int docarp(void)
 #endif
         if (received_signal != 0) {
             int flag = received_signal;
-	    
+        
             received_signal = 0;
             switch (flag) {
-	    case 1:
-		logfile(LOG_INFO, "%s on %s id %d", 
-			(sc.sc_state == BACKUP ? "BACKUP" : "MASTER"),
-			interface, sc.sc_vhid);
-		break;
-	    case 2:
+        case 1:
+        logfile(LOG_INFO, "%s on %s id %d", 
+            (sc.sc_state == BACKUP ? "BACKUP" : "MASTER"),
+            interface, sc.sc_vhid);
+        break;
+        case 2:
 #ifdef DEBUG
-		logfile(LOG_DEBUG, "Caught signal (USR2) considering going down");
+        logfile(LOG_DEBUG, "Caught signal (USR2) considering going down");
 #endif
-		if (sc.sc_state != BACKUP) {
-		    carp_set_state(&sc, BACKUP);
-		    sleep(3); /* hold up a sec... */
-		    carp_setrun(&sc, 0); /* now listen for 3 heartbeats, as usual */
-		    continue;
-		}
-		break;
+        if (sc.sc_state != BACKUP) {
+            carp_set_state(&sc, BACKUP);
+            sleep(3); /* hold up a sec... */
+            carp_setrun(&sc, 0); /* now listen for 3 heartbeats, as usual */
+            continue;
+        }
+        break;
             }
         }
-	
+    
         nfds = poll(pfds, (nfds_t) 1, sc.sc_advbase * 1000);
         if (nfds == -1) {
             if (errno == EINTR) {
